from dagster import Definitions
from dagster_components import (
    Component,
    ComponentRegistry,
    ComponentLoadContext,
    build_defs_from_toplevel_components_folder,
    component,
)

@component(name="{{ name }}")
class {{ component_type_class_name }}(Component):

    @classmethod
    def generate_files(cls) -> None:
        ...

    def build_defs(self, load_context: ComponentLoadContext) -> Definitions:
        ...
