[project]
name = "{{ project_name }}"
requires-python = ">=3.9,<3.13"
version = "0.1.0"
dependencies = [
    "dagster",
    "dagster-components[sling,dbt]",
    "dagster-embedded-elt",
    "dagster-dbt",
    "sling-mac-arm64",
]

[project.optional-dependencies]
dev = [
    "dagster-webserver",
]

[project.entry-points]
"dagster.components" = { {{ project_name }} = "{{ project_name }}.lib"}

[build-system]
requires = ["setuptools"]
build-backend = "setuptools.build_meta"

[tool.dagster]
module_name = "{{ project_name }}.definitions"
project_name = "{{ project_name }}"

[tool.setuptools.packages.find]
exclude=["{{ project_name }}_tests"]

{{ uv_sources }}
